package hu.se.app08;

import java.security.Key;
import java.util.Arrays;
import java.util.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

/**
 * Class under test.
 * 
 * Tests should check whether a text once encrypted and then decrypted
 * corresponds to the initial text.
 */
public class TextSecureApp {

	private SecurityKeys keys;
	private boolean highSecurityLevel;

	/**
	 * Instantiates the app.
	 * 
	 * @param keys
	 *            the security keys to be used.
	 * @param highSecurityLevel
	 *            <code>true</code> if a high security level should be used
	 *            (stronger encryption), <code>false</code> if a low security level
	 *            should be used.
	 */
	public TextSecureApp(SecurityKeys keys, boolean highSecurityLevel) {
		this.keys = keys;
		this.highSecurityLevel = highSecurityLevel;
	}

	/**
	 * Enrypts the given text.
	 * 
	 * @param text
	 *            the given text
	 * @return the text but encypted
	 */
	public String encryptText(String text) {
		try {
			String keyStr;
			byte[] keyByt;
			Key key;
			Cipher cipher;
			if (this.highSecurityLevel) {
				keyStr = this.keys.getSecretKey();
				keyByt = keyStr.getBytes("UTF-8");
				// just use the first 128 bits
				keyByt = Arrays.copyOf(keyByt, 16);
				key = new SecretKeySpec(keyByt, "AES");
				cipher = Cipher.getInstance("AES");
			} else {
				keyStr = this.keys.getLowSecretKey();
				keyByt = keyStr.getBytes("UTF-8");
				// just use the first 64 bits
				keyByt = Arrays.copyOf(keyByt, 8);
				key = new SecretKeySpec(keyByt, "DES");
				cipher = Cipher.getInstance("DES");
			}
			// encrypt the text
			cipher.init(Cipher.ENCRYPT_MODE, key);
			byte[] encrypted = cipher.doFinal(text.getBytes());
			String encryptedText = Base64.getEncoder().encodeToString(encrypted);
			return encryptedText;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

	}

	/**
	 * Decrypts the given encrypted text.
	 * 
	 * @param encrytpedText
	 *            the encrypted text
	 * @return the text but decrypted
	 */
	public String decryptText(String encrytpedText) {
		try {

			byte[] crypted = Base64.getDecoder().decode(encrytpedText);
			String keyStr;
			byte[] keyByt;
			Key key;
			Cipher cipher;
			if (this.highSecurityLevel) {
				keyStr = this.keys.getSecretKey();
				keyByt = keyStr.getBytes("UTF-8");
				// just use the first 128 bits
				keyByt = Arrays.copyOf(keyByt, 16);
				// Create key and cipher
				key = new SecretKeySpec(keyByt, "AES");
				cipher = Cipher.getInstance("AES");
			} else {
				keyStr = this.keys.getLowSecretKey();
				keyByt = keyStr.getBytes("UTF-8");
				// just use the first 64 bits
				keyByt = Arrays.copyOf(keyByt, 8);
				// Create key and cipher
				key = new SecretKeySpec(keyByt, "DES");
				cipher = Cipher.getInstance("DES");
			}
			// decrypt the text
			cipher.init(Cipher.DECRYPT_MODE, key);
			String decrypted = new String(cipher.doFinal(crypted));
			return decrypted;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

}
